from nn_wtf.tests.neural_network_graph_test import NeuralNetworkGraphTest
from nn_wtf.tests.images_labels_data_set_test import ImagesLabelsDataSetTest
from nn_wtf.tests.input_data_test import InputDataTest

import unittest

__author__ = 'Lene Preuss <lene.preuss@gmx.net>'

unittest.main()
